unit MainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ScoreL;

type
  TMainForm = class(TForm)
    ScoreList: TScoreList;
    StringGrid: TStringGrid;
    ShowButton: TButton;
    ScoreEdit: TEdit;
    Label1: TLabel;
    CheckButton: TButton;
    GreatScoreGroupBox: TGroupBox;
    NameEdit: TEdit;
    Label2: TLabel;
    NumberLabel: TLabel;
    procedure ScoreListNoFile(Sender: TObject);
    procedure ShowButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CheckButtonClick(Sender: TObject);
    procedure NameEditExit(Sender: TObject);
    procedure NameEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
	sgPlaceIndex=0;
	sgNameIndex=1;
	sgDateIndex=2;
	sgTimeSecsIndex=3;
	sgScoreIndex=4;

procedure TMainForm.FormCreate(Sender: TObject);
begin
    StringGrid.Cells[sgPlaceIndex, 0]:= 'Place number';
    StringGrid.Cells[sgNameIndex, 0]:= 'Name';
    StringGrid.Cells[sgDateIndex,0]:= 'Date';
    StringGrid.Cells[sgTimeSecsIndex,0]:= 'TimeSecs';
    StringGrid.Cells[sgScoreIndex,0]:= 'Score';
		StringGrid.RowCount:= ScoreList.MaxScores+1;
end;

procedure TMainForm.ScoreListNoFile(Sender: TObject);

	procedure CreateScore(const InitName: string; const InitDate: string;
		const InitPoints: longint);
	var ScoreListItem: TScoreListItem;
	begin
		ScoreListItem:=TScoreListItem.Create;
		with ScoreListItem do begin
			PlayerName:= InitName;
			Points:= InitPoints;
			Date:= InitDate;
			TimeSecs:=0;
		end;
		(Sender as TScoreList).InsertScore(ScoreListItem);
	end;
begin
	with (Sender as TScoreList) do
	begin
		CreateScore('DMagic','',10000);
		CreateScore('DMagic','',9000);
		CreateScore('DMagic','',8000);
		CreateScore('DMagic','',7000);
		CreateScore('DMagic','',6000);
		CreateScore('DMagic','',5000);
		CreateScore('DMagic','',4000);
		CreateScore('DMagic','',3000);
		CreateScore('DMagic','',2000);
		CreateScore('DMagic','',1000);
	end;
end;

procedure TMainForm.ShowButtonClick(Sender: TObject);
var
  i: integer;
  ScoreListItem: TScoreListItem;
  LastScore: longint;
  PlaceNumber: integer;
begin
  ScoreList.Lock;
  try
    PlaceNumber:= 0;
    LastScore:= -1;
    for i:=0 to ScoreList.MaxScores-1 do
		begin
      if ScoreList.ScoresCount>i then
			begin
        if ScoreList.Scores[i].Points <> LastScore then
					Inc(PlaceNumber); // It's here for a good reason - if two scores are the same!
				StringGrid.Cells[sgPlaceIndex, i+1]:= '#'+IntToStr(PlaceNumber);
				StringGrid.Cells[sgNameIndex, i+1]:= ScoreList.Scores[i].PlayerName;
				StringGrid.Cells[sgDateIndex, i+1]:= ScoreList.Scores[i].Date;
        StringGrid.Cells[sgTimeSecsIndex, i+1]:=
					IntToStr(ScoreList.Scores[i].TimeSecs);
				StringGrid.Cells[sgScoreIndex, i+1]:=
					IntToStr(ScoreList.Scores[i].Points);
        LastScore:= ScoreList.Scores[i].Points;
      end
      else begin
        Inc(PlaceNumber);
				StringGrid.Cells[0, i+1]:= '#'+IntToStr(PlaceNumber);
				StringGrid.Cells[1, i+1]:= '  -  -  -';
      end;
    end;
  finally
    ScoreList.Unlock;
  end;
end;

procedure TMainForm.CheckButtonClick(Sender: TObject);

  function GetTodaysDate: string;

    function Int2StrPad0(N: LongInt; Len: Integer): String;
    var
      S : String;
    begin
      Str(N:0,S);
      while Length(S) < Len do
        S := '0' + S;
      Int2StrPad0 := S;
    end;

  var
    y, m, d, dow : Word;
    Year: String[255];
  begin
    DecodeDate(Date, y,m,d);
    Year := Int2StrPad0(y, 4);
    Delete(Year, 1, 2);
    GetTodaysDate := Int2StrPad0(d, 2) + '/' + Int2StrPad0(m, 2) + '/' + Year;
  end;

var
	ScoreListItem: TScoreListItem;
	ScoreInserted: boolean;
	Score: integer;
begin
	Score:= StrToInt(ScoreEdit.Text);
	ScoreInserted:= false;
  ScoreList.Lock;
  try
    if (Score>0) and
      (ScoreList.GetPlace(Score)<>-1) then
    try
      GreatScoreGroupBox.Visible:= true;
      NumberLabel.Caption:= IntToStr(ScoreList.GetPlace(Score)+1)+'. place';

      // Input name...
      with NameEdit do begin
        SelectAll;
        Visible:= true;
        SetFocus;
        while Visible do
					Application.Processmessages;
      end;

      ScoreListItem:=TScoreListItem.Create;
      ScoreListItem.PlayerName:=NameEdit.Text;
      ScoreListItem.Points:=Score;
      ScoreListItem.Date:= GetTodaysDate;
      ScoreList.InsertScore(ScoreListItem);
      // Show the scorelist...
      ScoreInserted:=true;
		finally
      GreatScoreGroupBox.Visible:= false;
    end;
  finally
    ScoreList.Unlock;
  end;
	// If new score then update view...
	if ScoreInserted then
		ShowButtonClick(Self);
end;

procedure TMainForm.NameEditExit(Sender: TObject);
begin
	(Sender as TEdit).Visible:=false;
end;

procedure TMainForm.NameEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
	with Sender as TEdit do
	begin
		case Key of
			VK_RETURN:
				Visible:= false;
		end;
	end;
end;

end.
